#!/bin/bash
#
# Copyright (c) 2023 VMware Carbon Black, all rights reserved
#

function truncate_hinting_data() {
  if [[ $MIGRATOR_TRUNCATE_DATA == "true" ]]; then
    echo "$(date) Truncating Hinting data"
    psql "postgresql://$PGUSER:$PGPASSWORD@$PGHOST:$PGPORT/$PGDATABASE?sslmode=prefer" --echo-errors -c \
      "TRUNCATE TABLE devices, device_groups, organizations, environments;"

    echo "$(date) Flushing Hinting cache data"
    redis-cli -h $REDIS_CLUSTER_ENDPOINT --tls -a $REDIS_AUTH_TOKEN FLUSHALL
  else
    echo "$(date) Hinting data is NOT being truncated"
  fi
}

function create_cbd_tables() {
  echo "$(date) Creating temporary CBD tables"
  psql "postgresql://$PGUSER:$PGPASSWORD@$PGHOST:$PGPORT/$PGDATABASE?sslmode=prefer" -f cbd_data_setup.sql --quiet --echo-errors
}

function copy_all_cbd_data() {
  # Multiple Envs (e.g., prod01, prod02, prod05, and eap01 are all in prod05).
  if [[ -v CBDPGHOST_MULTI_ENV ]]; then
    env_count=$(echo $CBDPGHOST_MULTI_ENV | jq '.|length')
    for i in $(eval echo "{0..$(($env_count-1))}")
    do
      env=$(echo $CBDPGHOST_MULTI_ENV | jq '.|keys['$i']')
      echo "$(date) Migrating CBD data for $env"

      cluster_endpoint=$(echo $CBDPGHOST_MULTI_ENV | jq -r '( nth('$i'; .[]).clusterReaderEndpoint )')
      database_name=$(echo $CBDPGHOST_MULTI_ENV | jq -r '( nth('$i'; .[]).databaseName )')

      user=$(echo $CBDPG_CREDENTIALS_MULTI_ENV | jq -r --arg env $env '. | to_entries[] |  select(.key=='$env').value.username')
      password=$(echo $CBDPG_CREDENTIALS_MULTI_ENV | jq -r --arg env "$env" '. | to_entries[] |  select(.key=='$env').value.password')
      copy_cbd_data $cluster_endpoint $user $password $database_name
    done
  else
    # Single ENV
    echo "$(date) Migrating CBD data"
    copy_cbd_data $CBDPGHOST $CBDPGUSER $CBDPGPASSWORD $CBDPGDATABASE
  fi

  create_cbd_indices
}

function copy_cbd_data() {
  echo "$(date) Copying data from cbd database"
  
  #
  # Note that we check that a device_group is assigned to an org_domain as otherwise it is considered de-registered.
  # We use this JOIN for devices and device_groups. Similarly for organizations we use a JOIN to verify they are assigned
  # to an org_domain.
  #
  echo "$(date) Copying devices where org_key is not NULL"
  psql "postgresql://$2:$3@$1:$CBDPGPORT/$4?sslmode=prefer" --echo-errors -c \
        "COPY (select device.device_id, device.org_key, device.change_timestamps from device \
          JOIN device_group_device on device.device_id=device_group_device.device_id \
          JOIN device_group on device_group_device.device_group_id=device_group.device_group_id \
          JOIN org_domain on device_group.org_id=org_domain.org_id \
          WHERE device.org_key is not NULL) to stdout" | \
        psql "postgresql://$PGUSER:$PGPASSWORD@$PGHOST:$PGPORT/$PGDATABASE?sslmode=prefer" --echo-errors -c \
          "COPY cbd_device (device_id, org_key, change_timestamps) from stdin"

  echo "$(date) Copying devices where org_key is NULL"
  psql "postgresql://$2:$3@$1:$CBDPGPORT/$4?sslmode=prefer" --echo-errors -c \
        "COPY (select device.device_id, organization.org_key, device.change_timestamps from device \
          JOIN device_group_device on device.device_id=device_group_device.device_id \
          JOIN device_group on device_group_device.device_group_id=device_group.device_group_id \
          JOIN org_domain on device_group.org_id=org_domain.org_id \
          JOIN organization on org_domain.org_id=organization.org_id \
          WHERE device.org_key is NULL) to stdout" | \
        psql "postgresql://$PGUSER:$PGPASSWORD@$PGHOST:$PGPORT/$PGDATABASE?sslmode=prefer" --echo-errors -c \
          "COPY cbd_device (device_id, org_key, change_timestamps) from stdin"


  echo "$(date) Copying device groups"
  psql "postgresql://$2:$3@$1:$CBDPGPORT/$4?sslmode=prefer" --echo-errors -c \
       "COPY (select device_group_id, organization.org_key, device_group.change_timestamps, device_group.name from device_group \
         JOIN org_domain on device_group.org_id=org_domain.org_id \
         JOIN organization on org_domain.org_id=organization.org_id \
         WHERE device_group.change_timestamps is not NULL) to stdout" | \
       psql "postgresql://$PGUSER:$PGPASSWORD@$PGHOST:$PGPORT/$PGDATABASE?sslmode=prefer" --echo-errors -c \
         "COPY cbd_device_group (device_group_id, org_key, change_timestamps, name) from stdin"


  echo "$(date) Copying organizations"
  psql "postgresql://$2:$3@$1:$CBDPGPORT/$4?sslmode=prefer" --echo-errors -c \
       "COPY (select org_key, kill_switch_on, change_timestamps, deployment_type_change_timestamps, ransomware_config from organization \
         JOIN org_domain on organization.org_id=org_domain.org_id \
         WHERE change_timestamps is NOT NULL OR deployment_type_change_timestamps is not NULL OR ransomware_config is not NULL OR kill_switch_on is TRUE) to stdout" | \
       psql "postgresql://$PGUSER:$PGPASSWORD@$PGHOST:$PGPORT/$PGDATABASE?sslmode=prefer" --echo-errors -c \
         "COPY cbd_organization (org_key, kill_switch_on, change_timestamps, deployment_type_change_timestamps, ransomware_config) from stdin"

  if [[ $? -eq 0 ]]; then
    echo -e "\n$(date) Done copying CBD data"
  else
    echo -e "\n$(date) Failed to copy CBD data"
    exit 1
  fi
}

function create_cbd_indices() {
  echo "$(date) Creating index in cbd_device"
  psql "postgresql://$PGUSER:$PGPASSWORD@$PGHOST:$PGPORT/$PGDATABASE?sslmode=prefer" --echo-errors -c \
    "CREATE INDEX device_id_idx on cbd_device (device_id);"

  echo "$(date) Creating index in cbd_device_group"
  psql "postgresql://$PGUSER:$PGPASSWORD@$PGHOST:$PGPORT/$PGDATABASE?sslmode=prefer" --echo-errors -c \
    "CREATE INDEX device_group_id_idx on cbd_device_group (device_group_id);"

  echo "$(date) Creating index in cbd_organization"
  psql "postgresql://$PGUSER:$PGPASSWORD@$PGHOST:$PGPORT/$PGDATABASE?sslmode=prefer" --echo-errors -c \
    "CREATE INDEX org_key_idx on cbd_organization (org_key);"
}

function delete_cbd_data() {
    echo "$(date) Deleting temporary CBD data"
    psql "postgresql://$PGUSER:$PGPASSWORD@$PGHOST:$PGPORT/$PGDATABASE?sslmode=prefer" --echo-errors -c "DROP table IF EXISTS cbd_device cascade;"
    psql "postgresql://$PGUSER:$PGPASSWORD@$PGHOST:$PGPORT/$PGDATABASE?sslmode=prefer" --echo-errors -c "DROP table IF EXISTS cbd_device_group cascade;"
    psql "postgresql://$PGUSER:$PGPASSWORD@$PGHOST:$PGPORT/$PGDATABASE?sslmode=prefer" --echo-errors -c "DROP table IF EXISTS cbd_organization cascade;"
}

truncate_hinting_data

# Ensure we start with a clean slate by first deleting the temporary tables
if [[ "$RUN_MIGRATOR" == "true" ]]; then
  delete_cbd_data
  create_cbd_tables
  copy_all_cbd_data
fi

/migrator

delete_cbd_data

echo "$(date) Done migrating data"
